#!/usr/bin/perl
#
# Script to compare user-generated output files in $CSPP_NUCAPS_IASI_HOME/work/
# with test output files provided in $CSPP_NUCAPS_IASI_HOME/data/metopB/output/
#
# Uses only Perl modules that are part of the v5.006 standard install.
#
# Pre-requisites:
# CSPP_NUCAPS-IASI distribution in directory $CSPP_NUCAPS_IASI_HOME
#
# James E Davies, SSEC/UW-Madison,  June 2018
#
#
# NOTE:  This script only does rudimentary checking that it is comparing
# apples with apples.  If you process the test data with --mdrstart set
# to other than the default value of 'M', you will get a failed compare
# because test output granules and the newly generated ones are mis-aligned.


# Pragmata
# ````````
use 5.006;
use strict;
use warnings;


# Load Perl modules - standard modules only
# `````````````````````````````````````````
use File::Spec;
use File::Basename;
use I18N::Langinfo qw(langinfo CODESET);


# Global variables are all UPPERCASE throughout
# `````````````````````````````````````````````
our $MAIN = basename($0);
defined $ENV{CSPP_NUCAPS_IASI_HOME} or die "$MAIN\:: CSPP_NUCAPS_IASI_HOME not set.\nAbort";


# $NCDUMP points to supplied static-compiled ncdump (netcdf library version 4.3.0)
# ````````````````````````````````````````````````````````````````````````````````
our $NCDUMP = File::Spec -> catfile($ENV{'CSPP_NUCAPS_IASI_HOME'}, 'bin', 'ncdump');
(-e $NCDUMP) or die "$MAIN\:: $NCDUMP not found.\nAbort";
(-x $NCDUMP) or die "$MAIN\:: $NCDUMP not executable.\nAbort";


# Assume that files were processed by user in $PWD
# ````````````````````````````````````````````````
my @user_ccr = (sort(glob("IASI_CCR_*.nc")));
my @user_edr = (sort(glob("IASI_L02_*.nc")));


# The matching verification files are in $ENV{CSPP_NUCAPS_HOME}/data/output_npp
# `````````````````````````````````````````````````````````````````````````````
my @veri_ccr = (sort(glob("$ENV{CSPP_NUCAPS_IASI_HOME}/data/metopB/output/IASI_CCR_*.nc")));
my @veri_edr = (sort(glob("$ENV{CSPP_NUCAPS_IASI_HOME}/data/metopB/output/IASI_L02_*.nc")));


# Define pass/fail flag
# `````````````````````
my $passed = 1;


# Were the CCR files generated?
# `````````````````````````````
($#user_ccr < 0) and do {
  my $user = $#user_ccr + 1;
  print "Number of user_ccr files = $user\n";
  warn "WARNING: Cannot find any user CCR files, jumping to L02_Quality_Flag\n";
  goto EDR; };


# Do I have matchning numbers of CCR files in user and verification sets?
# ```````````````````````````````````````````````````````````````````````
#CCR:
($#user_ccr == $#veri_ccr) or do {
  my $user = $#user_ccr + 1;
  my $veri = $#veri_ccr + 1;
  print "Number of user_ccr files = $user\n";
  print "Number of veri_ccr files = $veri\n";
  warn "WARNING: Mismatched user and verification CCR files, jumping to L02 files\n";
  goto EDR; };


# CCR:Quality_Flag
# ````````````````
CCR_Quality_Flag:
print "CCR_Quality_Flag:\n";
my @user = as_numeric_vector('Quality_Flag', @user_ccr);
my @veri = as_numeric_vector('Quality_Flag', @veri_ccr);
($#user == $#veri) or do {
  warn "WARNING: Mismatched CCR #Quality_Flag, jumping to CCR_Latitude\n"; 
  goto CCR_Latitude; };
my $count = 0;
for (my $i=0; $i<=$#veri; $i++) {
  ($user[$i] != $veri[$i]) and $count++; 
}
my $pct = $count / $#veri * 100.0;
($pct > 0.0001) and do {
  warn "WARNING: More than 0.0001% of CCR Quality_Flag different.\n";
  $passed = 0; };


# CCR:Latitude
# ````````````
CCR_Latitude:
print "CCR_Latitude:\n";
@user = as_numeric_vector('Latitude', @user_ccr);
@veri = as_numeric_vector('Latitude', @veri_ccr);
($#user == $#veri) or do {
  warn "WARNING: Mismatched CCR #Latitude, jumping to CCR_Longitude\n"; 
  goto CCR_Longitude; };
$count = 0;
for (my $i=0; $i<=$#veri; $i++) {
  (abs($user[$i]-$veri[$i]) > 0.0001) and $count++; 
}
$pct = $count / $#veri * 100.0;
($pct > 1) and do {
  warn "WARNING: More than 1% of CCR Latitude differ by more than 0.0001 deg.\n";
  $passed = 0; };


# CCR:Longitude
# `````````````
CCR_Longitude:
print "CCR_Longitude:\n";
@user = as_numeric_vector('Longitude', @user_ccr);
@veri = as_numeric_vector('Longitude', @veri_ccr);
($#user == $#veri) or do {
  warn "WARNING: Mismatched CCR #Longitude, jumping to EDR\n"; 
  goto EDR; };
$count = 0;
for (my $i=0; $i<=$#veri; $i++) {
  (abs($user[$i]-$veri[$i]) > 0.0001) and $count++; 
}
$pct = $count / $#veri * 100.0;
($pct > 1) and do {
  warn "WARNING: More than 1% of CCR Longitude differ by more than 0.0001 deg.\n";
  $passed = 0; };


# EDE file checks
# ```````````````
EDR:
($#user_edr == $#veri_edr) or do {
  my $user = $#user_edr + 1;
  my $veri = $#veri_edr + 1;
  print "Number of user_edr files = $user\n";
  print "Number of veri_edr files = $veri\n";
  warn "WARNING: Mismatched user and verification L02 files, exit\n";
  exit(1); };

# L02:Quality_Flag
# ````````````````
L02_Quality_Flag:
print "L02_Quality_Flag:\n";
@user = as_numeric_vector('Quality_Flag', @user_edr);
@veri = as_numeric_vector('Quality_Flag', @veri_edr);
($#user == $#veri) or do {
  warn "WARNING: Mismatched L02 #Quality_Flag, jumping to L02_Temperature\n"; 
  goto L02_Temperature; };
$count = 0;
for (my $i=0; $i<=$#veri; $i++) {
  ($user[$i] != $veri[$i]) and $count++; 
}
$pct = $count / $#veri * 100.0;
($pct > 0.0001) and do {
  warn "WARNING: More than 0.0001% of L02 Quality_Flag different.\n";
  $passed = 0; };


# L02:Temperature
# ```````````````
L02_Temperature:
print "L02_Temperature:\n";
@user = as_numeric_vector('Temperature', @user_edr);
@veri = as_numeric_vector('Temperature', @veri_edr);
($#user == $#veri) or do {
  warn "WARNING: Mismatched L02 #Temperature, jumping to L02_H2O_MR\n"; 
  goto L02_H2O_MR; };
$count = 0;
for (my $i=0; $i<=$#veri; $i++) {
  (abs($user[$i]-$veri[$i]) > 0.01) and $count++; 
}
$pct = $count / $#veri * 100.0;
($pct > 1) and do {
  warn "WARNING: More than 1% of L02 Temperature differ by more than 0.01 deg.\n";
  $passed = 0; };


# L02:H2O_MR
# ```````````
L02_H2O_MR:
print "L02_H2O_MR:\n";
@user = as_numeric_vector('H2O_MR', @user_edr);
@veri = as_numeric_vector('H2O_MR', @veri_edr);
($#user == $#veri) or do {
  warn "WARNING: Mismatched CCR #H2O_MR, jumping to L02_SO2_MR\n"; 
  goto L02_SO2_MR; };
$count = 0;
for (my $i=0; $i<=$#veri; $i++) {
  (abs($user[$i]-$veri[$i]) > 0.000001) and $count++; 
}
$pct = $count / $#veri * 100.0;
($pct > 1) and do {
  warn "WARNING: More than 1% of L02 H2O_MR differ by more than 0.000001 g/g/.\n";
  $passed = 0; };


# L02:SO2_MR
# ``````````
L02_SO2_MR:
print "L02_SO2_MR:\n";
@user = as_numeric_vector('SO2_MR', @user_edr);
@veri = as_numeric_vector('SO2_MR', @veri_edr);
($#user == $#veri) or do {
  warn "WARNING: Mismatched CCR #SO2_MR, jumping to DONE\n"; 
  goto DONE; };
$count = 0;
for (my $i=0; $i<=$#veri; $i++) {
  (abs($user[$i]-$veri[$i]) > 0.000001) and $count++; 
}
$pct = $count / $#veri * 100.0;
($pct > 1) and do {
  warn "WARNING: More than 1% of L02 SO2_MR differ by more than 0.000001 ppb.\n";
  $passed = 0; };


# All done, report PASSED or FAILED
# `````````````````````````````````
DONE:
if ($passed) {
  print "+++++ PASSED +++++\n";
  exit(0);
} else {
  print "+++++ FAILED ++++++\n";
  exit(1);
}

# ---------------------
  sub as_numeric_vector
# =====================
# @data = as_numeric_vector($varname, @granule_filenames);
  {
    # @data is a vector of numbers that comprise the data from
    # all @granule_filenames for specific $varname.
    my $varname = shift;
    my @files = sort(@_);
    my $name = (caller(0))[3];
    $name =~ s/main/$MAIN/g;
    my @data = ();
    foreach my $fullname (@files) {
      my $key = (split('\.', basename($fullname)))[0];
      my $cmd = "$NCDUMP -v $varname $fullname |";
      open(my $fh, $cmd) or die "$name\: Could not run command $cmd\nAbort";
      while (<$fh>) { chomp; ( /data:/ ) and last; }
      my @buffer = ();
      while (<$fh>) { chomp; ( /}/ ) and last; push(@buffer, $_); }
      close($fh);
      foreach (@buffer) {
        my $value = $_;
        my @parts = split(/=/, $_);
        ($#parts == -1) and next;
        ($#parts ==  0 and /=/) and next;
        ($#parts ==  0) and $value = (split(/;/, $parts[0]))[0];
        ($#parts ==  1) and $value = (split(/;/, $parts[1]))[0];
        @parts = split(/,/, $value);
        foreach (@parts) {
          my $value = $_;
          $value =~ s/^\s+|\s+$//g;
          (length($value)==0) and next;
          ($value eq '_') and $value = -999;
          push(@data, $value)
        }
      }
    }
    return @data;
  }
